#!/bin/bash

#
# File Name: db2ConfigureKerberos.sh
#
# Licensed Materials - Property of IBM
#
# 5724-T60 5608-A91
# (C) Copyright IBM Corp. 2021  All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication, or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#

##############################################################################
# This script will configure db2 with kerberos
##############################################################################

GID=`id -g`
message=""
UPPER=""
LOWER=""
KINIT_PATH=""
CURDIR=`pwd`
export TMPDIR=$CURDIR
OS=`uname`

printusage() {
echo "*************************************************************************
	 Usage:	 ./db2ConfigureKerberos.sh path_of_krb5.conf path_of_krb5.keytab kdc_server realmName db2ServiceName path_of_sqllib_folder
	 where path_of_krb5.conf is the path of krb5.conf file default is <SKLM_INSTALL_HOME>/kerberos
	       path_of_krb5.keytab is the path of the krb5.keytab file default is <SKLM_DATA_DIR>
		   kdc_server is the hostname of the kdc server
		   db2ServiceName is the name of db2 service registered with the kdc server
		   and path_of_sqllib is the path of the sqllib folder default is /home/sklmdb41/sqllib
		
		eg: ./db2ConfigureKerberos.sh /opt/IBM/SKLMV41/kerberos/krb5.conf /opt/IBM/WebSphere/AppServer/products/sklm/data/krb5.keytab kdc.ibm.com IBM.COM sklmdb41/test.ibm.com@IBM.COM /home/sklmdb41/sqllib
	 

Note:	 Please ensure to use this script for Linux and AIX OS only                               
							
********************************************************************************"
exit 1
}

if [ ! $6 ]; then
        printusage
fi

if [ $GID -eq 0  ]; then
        echo "Cannot run this operation as a root user. Only the process owner user can run this operation."
        exit 1        
fi

installKinit() {
	which kinit >> $TMPDIR/configKerberos.log
	KINIT_PATH=$(which kinit)
}

setupConfFileLinux() {
	if [ ! -f "$1" ]
	then
		echo "Conf file does not exist at the specified location."
		exit 1
	fi
	
	UPPER=`echo "$2" | awk '{print toupper($0)}'` 
	LOWER=`echo "$2" | awk '{print tolower($0)}'`
	
	sed -i "s/EXAMPLE\.COM/"$UPPER"/g" /$1
	sed -i "s/kerberos\.example\.com/"$3"/g" /$1
	sed -i "s/example\.com/"$LOWER"/g" /$1

	
}

configureDb2Linux() {

	echo 'export KRB5_KTNAME='$2'' >> ~/.bash_profile
	echo 'export KRB5_CONFIG='$1'' >> ~/.bash_profile
	. ~/.bash_profile
	db2set DB2ENVLIST="KRB5_KTNAME KRB5_CONFIG"
	db2 update dbm cfg using CLNT_KRB_PLUGIN IBMkrb5 
	db2 update dbm cfg using SRVCON_GSSPLUGIN_LIST IBMkrb5 
	db2 update dbm cfg using AUTHENTICATION KERBEROS
}

setupConfFileAIX() {
    if [ ! -f "$1" ]
    then
        echo "Conf file does not exist at the specified location."
        exit 1
    fi
    
    UPPER=`echo "$2" | awk '{print toupper($0)}'` 
    LOWER=`echo "$2" | awk '{print tolower($0)}'`
    
    sed "s/EXAMPLE\.COM/"$UPPER"/g" /$1 > temp
    mv temp $1
    sed "s/kerberos\.example\.com/"$3"/g" /$1 > temp
    mv temp $1
    sed "s/example\.com/"$LOWER"/g" /$1 > temp
    mv temp $1    
}

configureDb2AIX() {

    echo 'export KRB5_KTNAME='$2'' >> ~/.profile
    echo 'export KRB5_CONFIG='$1'' >> ~/.profile
    . ~/.profile
    db2set DB2ENVLIST="KRB5_KTNAME KRB5_CONFIG"
    db2 update dbm cfg using CLNT_KRB_PLUGIN IBMkrb5 
    db2 update dbm cfg using SRVCON_GSSPLUGIN_LIST IBMkrb5 
    db2 update dbm cfg using AUTHENTICATION KERBEROS
}

restartDb2() {
	"$KINIT_PATH" -k -t $1 $2
	db2stop force
	db2start
}

addKinitToDb2Service() {
    kinit_command=""$KINIT_PATH" -k -t $KRB5_KTNAME $2"
    if [ "${OS}" == "Linux" ] 
    then 
       sed -i '/#run kinit to get kerberos ticket/a '"${kinit_command}"'' $1/db2start.sh
       sed -i '/#run kinit to get kerberos ticket/a '"${kinit_command}"'' $1/db2stop.sh
    else        
       awk  -v command="${kinit_command}" '/run kinit to get kerberos ticket/{print;print command;next}1' /$1/db2start.sh > temp
       mv temp /$1/db2start.sh
       awk -v command="${kinit_command}" '/run kinit to get kerberos ticket/{print;print command;next}1' /$1/db2stop.sh > temp
       mv temp /$1/db2stop.sh
    fi

}


#Executing Main
if [ "${OS}" == "Linux" ] || [ "${OS}" == "AIX" ]
then
	installKinit
	if [ "$?" -ne 0 ]
	then
		message="\nError: kinit not found. \nInstall it on the system to proceed with the kerberos configuration. "
		echo -e "$message"
		exit 1
	fi
	
	FILE=$2
	if [ ! -f "$FILE" ]
	then
		message="\nKeytab file does not exist at the specified location. "
		echo -e "$message"
		exit 1
	fi
	
	if [ "${OS}" == "Linux" ] 
	then 
	   setupConfFileLinux $1 $4 $3
	else  
	   setupConfFileAIX $1 $4 $3
	fi
	if [ $? -ne 0 ]
	then
        echo -e "Error setting the config file."
        exit 1
    fi
    
	if [ "${OS}" == "Linux" ] 
	then 
	   configureDb2Linux $1 $2
	else 
	   configureDb2AIX $1 $2
	fi
	if [ $? -ne 0 ]
    then
        echo -e "Error setting the db2 with kerberos configuration."
        exit 1
    fi
	restartDb2 $2 $5
	if [ $? -ne 0 ]
    then
        echo -e "Error while restarting the db2."
        exit 1
    fi
    addKinitToDb2Service $6 $5
    if [ $? -ne 0 ]
    then
        echo -e "Error while adding kinit in the db2 service file."
        exit 1
    fi
	echo -e "Db2 kerberos Configuration completed."
	exit 0
	
else
	printusage
fi
	
